Option Explicit

   '******************************************************************
   ' FILENAME: VERIFY2K.BAS
   ' Y2K date handler and verification module for VB
   ' For dates formatted as **/**/** or **/**/****
   ' Handles dates from 1/1/0000 to 12/31/9999
   '
   ' Version      v1.21
   ' Original     1/8/1998
   ' Update       1/9/1998   added century independance
   ' Update       1/17/1998  updated to handle right aligned textboxes
   '                         updated various portions of code
   ' Update       1/25/1998  updated a comment under the year charts.
   '                         also updated some of the explanations.
   '
   ' Add this file as a *.BAS to your project
   '
   ' Written in VB3.
   ' Should work in VB4(16bit), may work in VB4 and VB5, but not tested.
   '
   ' NOTE - THIS HAS _NOT_ BEEN TESTED WITH RICH TEXT FORMATS -
   '
   ' 1998 by Dan Hinrichs
   ' hinrichs@alaska.net       70254.3513@compuserve.com
   ' PO Box 91172, Anchorage, AK  99509-1172 USA
   '
   ' FREEWARE -- the enclosed code may be used and/or published as is.
   ' Any alterations are the sole responsibility of the editor.
   '
   ' Liabilities - None       Royalies - None      Technical Support - None
   ' Guarantees  - None       Limitations - None
   '
   ' These comments are best viewed when using a fixed pitch font.
   '
   ' Use of this module and/or any derivitives are the responsibility
   ' of the person who chooses to use this collection of routines.
   '
   ' Feel free to re-write and reformat as necessary.
   ' In doing so, do not include the original author.
   '
   ' Check in the BASIC forum of CompuServe for any updates that may exist.
   ' Any updates will be posted under the same filename.
   '
   ' PURPOSE AND THEORY **************************************************
   '
   ' This module was started due to a problem associated in retrieving the
   ' value of the Year() function when given a TWO digit year.  From our
   ' experience, we have found that the Year() function "ONLY" returns
   ' 20th century (19**) values when given a TWO digit year. (Win-16bit)
   '
   ' Example:
   ' If System Clock = year 19**, Then Year(95) returns 1995 ( this is ok )
   ' If System Clock = year 20**, Then Year(05) returns 1905 ( go figure !!)
   '
   ' This module allows a user to enter in a variety of date formats
   ' like **.*.** or */*/** or *-*-**** or *,*,**.
   ' NOTE the date input may have one of several different types of
   ' delimiters, but there may only be ONE type of delimiter used.
   ' This module does NOT deal with TEXT type dates: ie. Dec 12, 1997,
   ' though with some work, it could be done.
   '
   ' Using the supplied routine, you can allow 2 and/or 4 digit years
   ' to be entered, with the routine interpreting the century for you.
   '
   ' A CROSSOVER flag exists to check at which point a rollover year
   ' is established.  If this routine is run in a new century, but
   ' before the CROSSOVER year, then all TWO digit years are compared
   ' to the CROSSOVER year, and adjustments are made accordingly.
   ' If this routine is run in a new century, but AFTER the CROSSOVER
   ' year, then ALL TWO DIGIT YEARS are adjusted to the current century.
   '
   ' This feature re-runs itself at every turn of 100 years, so that
   ' in the year 2100, users are presented with the same features.
   '
   ' Three digit years are corrected to four places with a leading zero.
   '
   ' Four digit years are passed through as four digits...
   ' ... BUT are returned in 2 or 4 digits depending on the setting
   ' of the Allow2DigYr% flag, and a stop flag within the routine.
   ' Search for " 4-GETS-YOU-4 " to find this stop flag.
   '
   ' EXAMPLES *************************************************************
   '
   '                  ----------current system year clock settings-------------
   '                        1995        |        2000        |        2005+
   '                  System2DigitYear  |  System2DigitYear  |  System2DigitYear
   '  Crossover%=5      > Crossover     |    < Crossover     |     >= Crossover
   '                                    |                    |
   '  Return2DigYr%=  False     True    |   False     True   |  False      True
   '   ----------------------------------------------------------------------------
   '  <entry>
   '  1/1        |  1/1/1995   1/1/95     1/1/2000   1/1/2000  1/1/2005   1/1/05
   '  1.1.100    |  1/1/0100   1/1/0100   1/1/0100   1/1/0100  1/1/0100   1/1/0100
   '  1-1-1000   |  1/1/1000   1/1/1000   1/1/1000   1/1/1000  1/1/1000   1/1/1000
   '  1,1,1850   |  1/1/1850   1/1/1850   1/1/1850   1/1/1850  1/1/1850   1/1/1850
   '             |
   '  1/01/04    |  1/1/1904   1/1/04     1/1/2004   1/1/04    1/1/2004   1/1/04
   '  1.1.05     |  1/1/1905   1/1/05     1/1/1905   1/1/1905  1/1/2005   1/1/05
   '  01/1/25    |  1/1/1925   1/1/25     1/1/1925   1/1/1925  1/1/2025   1/1/25
   '  01/01/1925 |  1/1/1925   1/1/1925   1/1/1925   1/1/1925  1/1/1925   1/1/1925
   '  1/1/2025   |  1/1/2025   1/1/2025   1/1/2025   1/1/2025  1/1/2025   1/1/2025
   '  1/1/35     |  1/1/1935   1/1/35     1/1/1935   1/1/1935  1/1/2035   1/1/35
   '  1/1/95     |  1/1/1995   1/1/95     1/1/1995   1/1/1995  1/1/2095   1/1/95
   '  1/1/2035   |  1/1/2035   1/1/2035   1/1/2035   1/1/2035  1/1/2035   1/1/2035
   '
   ' -- or if you COMMENT out a flag assignment, you can allow returns of two
   '     digits if the System Century is the same (Search for " 4-GETS-YOU-4 ")
   '
   '  1/1/1935   |  1/1/1935   1/1/35     1/1/1935   1/1/1935  1/1/1935   1/1/1935
   '  1/1/2035   |  1/1/2035   1/1/2035   1/1/2035   1/1/35    1/1/2035   1/1/35
   '
   '
   ' If the TWO DIGIT year of the COMPUTER SYSTEM CLOCK is >= the CROSSOVER%
   ' year, then ALL two digit years are returned in two and/or four digit
   ' years of the current century.  The two digit year adjustment feature
   ' stops making adjustments after the computer clock has passed the
   ' CROSSOVER date.
   '
   ' Though we have heard alot mentioned on 30 ~ 35 being a standard for
   ' a rollover year, we have opted to set the CROSSOVER% year at 5 for
   ' our own use. In thiry years, users may be wondering why their programs
   ' always make adjustments to the previous century!!!
   '
   ' Other options are to have the CROSSOVER% year obtained from ini files.
   '
   ' **********************************************************************
   ' Added features 1/9/98 -
   ' NOT CENTURY SPECIFIC - allows enabling/disabling of TWO digit year
   ' handling in any new century.
   '
   ' Contains a feature to allow the year handler to automatically stop
   ' adjusting two digit years after the CROSSOVER year has passed.
   '
   ' Contains programmer setting to allow/disallow returning TWO digit
   ' years for the current century, if given a FOUR digit year.
   '
   ' Checks if the current century is a **00 value (zero year), and then
   ' returns 4 digit to ASSURE the USER that it is indeed recording the
   ' current century. This mandatory return of 4 digits in the zero year
   ' only takes place when the current century is a zero year (**00).
   ' After the current system clock goes to a non-zero year, then the
   ' 4 digit required return of **00 reverts to any other flag settings.
   '
   ' Example:
   '     If Year(Now) = 2000 Then
   '        User typed 1/1/0
   '        If Return2DigYr% = ( True or False ) Then
   '           this routine automatically returns 1/1/2000.
   '
   '     If Year(Now) = 2001 Then
   '        User typed 1/1/0
   '        If Return2DigYr% = True
   '           this routine returns 1/1/00
   '        ElseIf Return2DigYr% = False Then
   '           this routine returns 1/1/2000
   '
   ' OF COURSE - THIS CAN ALL BE ELIMINATED !!! - YOU HAVE THE SOURCE CODE !
   '**********************************************************************
   ' UPDATE 1/18/98
   ' Updated to handle strings passed from RIGHT-ALIGNED textboxes.
   ' In VB3, the standard textbox can only be right aligned when then
   ' control is set in the MULTILINE mode.  In this mode, strings are
   ' passed with carriage return Chr$(13) and line feed Chr$(10)
   ' characters embedded in the string.
   ' This update removes these characters from the date string.
   '
   ' Also gave ability if user typed in 4 digit year of current
   ' century, then a 4 digit year would be returned, regardless
   ' of the Allow2DigYr% setting...
   ' This goes in hand with giving the user what they want... if they
   ' wanted to type a four digit year, then give it to them..!!
   ' This can be changed by commenting out a value assignment.
   '
   '***********************************************************************

Sub VerifyDate (Cntl As Control)
   
   '****************************************************************
   'CALLED BY: any calling routine...
   '
   'PARAMETERS: Cntl as Control
   '           This allows focus to return to the control
   '           in the event that the date was invalid
   '
   'RETURNS:   Formatted date in m/d/yy or m/d/yyyy (set by flags)
   '           If an error occurs, then a messagebox is presented
   '           and focus is set to the control without changes.
   '
   'EXAMPLE:   Place this in your "Lost Focus" event...
   '
   '           VerifyDate Text3  'it's as simple as this !!
   '
   'NOTE: If you send a blank date (NULL), then the routine exits.
   '        Code around this if you need to require a date.
   '        See associated code to setup as a required entry !!
   '***************************************************************
   
   On Error GoTo VerifyDate_SysError
   
   'declarations
   Dim Allow2DigYr%     'allow current century as TWO DIGIT
   Dim CenturyVal%      'container for CENTURY variable
   Dim NL As String * 1 'string variable for NL constant
   Dim i As Integer     'temp variable used in FOR loops
   Dim TempDate$        'our temp variable to test on
   Dim Msg$             'for error messages returned to user
   Dim ErrorFlag%       'integer flag to show where we errored
   Dim CROSSOVER%       'year of crossover for entering in TWO digit years
   Dim DayVal%          'integer of day value
   Dim MonthVal%        'integer of month value
   Dim YearVal%         'integer of year value
   Dim YearTxt$         'text of year - allows us to check if 2 or 4 digits
                           'were entered by calling proceedure
   Dim YearEntered%     'integer flag to show if the year was sent
                           'or if we need to add the current century/year
   Dim OldPos%          'integer for progress of DO/WHILE scan of date
   Dim CurPos%          'integer for progress of DO/WHILE scan of date
   Dim DateFld%         'used in our parsing routine to track which field
   Dim AscDelimiter%    'ASCII number for storing our delimiter
   Dim CharVal%         'Used to build date string character by character
   Dim Delimiter$       'predefine our returned date delimiter

   Const FLD_MONTH = 1  'Constants for parsing routine
   Const FLD_DAY = 2
   Const FLD_YEAR = 3
   
   
   'preset necessary variables
   ErrorFlag% = 0       'start out our error routine at 0
   Delimiter$ = "/"     'predefine our returned date delimiter
   NL = Chr$(10)        'predefine NewLine character for messages

   
   'Allow2DigYr% is used to allow the return of 2 digit years.
   'Assign this somewhere in your program or use an IF statement
   'to check a flag somewhere.
   'This could be set as a parameter setting.
   'This flag is evaluated close to end of this routine.
   'Look in Select Case Len(YearString) = 4 digits
   'as the value of this flag may change if user gave a 4 digit year.

   'Allow2DigYr% = Not frmEntry!mnuSettings(2).Checked
   Allow2DigYr% = True
   

   'The CROSSOVER year is the dividing year for TWO digit year numbers.
   'Any TWO digit year entries greater than the CROSSOVER will default
   'to the PREVIOUS century, with entries less than the CROSSOVER
   'defaulting to the CURRENT century.

   'You could move this somewhere else in your program's init routines.
   'If you require all TWO digit years to only use the current year,
   'then set this value to 99.
   
   CROSSOVER% = 5    '35   'Tried both 5 and 35... your choice
                           'if users haven't gotten used to the previous
                           'century being 4 digits, then increase..

   'If you wanted to make the CROSSOVER a parameter (or global)
   'and wanted the user to select the year, you might UNCOMMENT out
   'the following IF statement.

   'Just in case a user selected ZERO to mean never change century...
   'you may want to add/remove this as needed.
   
   'If Crossover% = 0 Then Crossover% = 99

   '** BEGIN WORK HERE **********************************

   ErrorFlag% = 100

   'Assign the data to our working container.
   'Take care of issues with RIGHT ALIGNED TEXTBOXES...
   'Strip out all CARRIAGE RETURNS, LINE FEEDS and SPACES

   'Check to see if DATE structure is valid
   'Verify only numeric type of date structure
   'Only allow numeric and the characters /,.-
   'Find and buffer the delimiter used by the user
   'We do NOT allow LONG date descriptions - like "Dec. 12"

   'updated 1/18/1998
   For i = 1 To Len(Cntl)
      CharVal% = Asc(Mid$(Cntl, i, 1))
      Select Case CharVal%
         Case 48 To 57
            'all numbers are permissable
            TempDate$ = TempDate$ & Chr$(CharVal%)

         Case 44 To 47
            'check for date delimiters
            'buffer the first delimiter found, and stay with it.
            'only allow for one type of delimiter
            'characters allowed ,-./
            
            If Not AscDelimiter% > 0 Then
               'set ascii date delimiter
               AscDelimiter% = CharVal%
            Else
               If Not CharVal% = AscDelimiter% Then
                  'user has used more than one type of separator
                  ErrorFlag% = 101
                  GoTo VerifyDate_Error
               End If
            End If

            TempDate$ = TempDate$ & Chr$(CharVal%)

         'test for R-ALIGN STRUCTURES
         'added 1/18/1998
         Case 10, 13, 32
            'this is the result of a R-ALIGNED string being passed...
            'skip assigning the character...
            'also strip out spaces

         Case Else
            'other date structure possibly typed in
            'this routine does not handle named dates
            'user could have typed in "Dec 12, 1995"
            ErrorFlag% = 102
            GoSub VerifyDate_Error
      End Select
   Next i

   'no point checking if there isn't anything sent
   '..this could also be setup for a parameter ( Required As Integer) !!

   If Not Len(TempDate$) > 0 Then
      GoTo VerifyDate_Exit
      '*** OR IF YOU REQUIRE AN INPUT ***
      'ErrorFlag% = 150     'also see error routine to catch this..
      'GoTo VerifyDate_Error
   End If
   
   ErrorFlag% = 200

   'error trap
   'if user did not use ANY delimiters, then error
   If Not AscDelimiter% > 0 Then
      ErrorFlag% = 201
      GoTo VerifyDate_Error
   End If
   
   ErrorFlag% = 300

   'seed the starting mode for our parsing utility
   DateFld% = FLD_MONTH
   OldPos% = 1
   CurPos% = 1
   
   'parsing utility for our date fields
   For DateFld% = FLD_MONTH To FLD_YEAR
      Do While Not CurPos% > Len(TempDate$)
         
         'temp catcher for R-ALIGNED strings with Chr$(10)-Chr$(13)
         'If Mid$(TempDate$, CurPos%, 1) = Chr$(13) Then Exit For

         'if the delimiter is found in the string, then
         'we move to the next field.
         'or if the pointer is at the last character, handle it.
         'Leave the error flags and check for OVERFLOWS

         If Mid$(TempDate$, CurPos%, 1) = Chr$(AscDelimiter%) Or CurPos% = Len(TempDate$) Then
            
            Select Case DateFld%
               Case FLD_MONTH
                  ErrorFlag% = 301
                  'if nothing, then missing month
                  If CurPos% = OldPos% Then GoTo VerifyDate_Error

                  MonthVal% = Val(Mid$(TempDate$, OldPos%, CurPos% - OldPos%))

                  CurPos% = CurPos% + 1
                  OldPos% = CurPos%
                  Exit Do

               Case FLD_DAY
                  ErrorFlag% = 302
                  'check to see if the pointer is on the last character
                  'if it is, then the year has not been entered
                  'and we need to pick up all characters since the month field
                  If CurPos% = Len(TempDate$) Then
                     DayVal% = Val(Mid$(TempDate$, OldPos%))
                  Else
                     If CurPos% = OldPos% Then GoTo VerifyDate_Error
                     DayVal% = Val(Mid$(TempDate$, OldPos%, CurPos% - OldPos%))
                  End If
                  
                  CurPos% = CurPos% + 1
                  OldPos% = CurPos%
                  Exit Do

               Case FLD_YEAR
                  ErrorFlag% = 303
                  'if we made it here, the string contains 'year' data
                  YearTxt$ = Mid$(TempDate$, OldPos%)

                  Exit Do

            End Select
         End If

         CurPos% = CurPos% + 1
      Loop
   Next DateFld%
   
   
   ErrorFlag% = 400

   'now, let's straighten out the year issues
   'if YEAR is 4 characters and >=2000, then CROSSOVER kicks in
   'this is century independant
   
   Select Case Len(YearTxt$)
      Case 0, Null   'no year entered
         YearTxt$ = Year(Now)

      Case 1, 2
         'If the current year that this program is running
         'is less than the crossover year (not century specific),
         'then we run our adjustment routine.
         'When the current year is >= than the crossover year,
         'the adjustment routine disengages
         'This allows us to not worry what century we are in..
         'we can have the crossover reactivate in 2000, 2100, 3000
         
         If (Year(Now) Mod 100) < CROSSOVER% Then
            'compare if typed year is less than the adjustment year
            If Val(YearTxt$) > CROSSOVER% Then
               YearVal% = (((Year(Now) \ 100) - 1) * 100) + Val(YearTxt$)
            Else
               YearVal% = ((Year(Now) \ 100) * 100) + Val(YearTxt$)
            End If

         Else  'our crossover adjustment is disabled >= the crossover year
               'and all two digit year entries default to current year
            YearVal% = ((Year(Now) \ 100) * 100) + Val(YearTxt$)
         End If

         YearTxt$ = CStr(YearVal%)

      Case 3
         YearTxt$ = CStr(Format(YearTxt$, "0000"))
      
      Case 4
         '" 4-GETS-YOU-4 "
         'MODIFIED 1/18/1998
         'If user typed a 4 digit year, then this flag sets to
         'require returning a 4 digit year, even in current century.
         'COMMENT OUT THIS VALUE ASSIGNMENT IF YOU WILL ALLOW
         'CURRENT CENTURY 4 DIGITS TO RETURN IN 2 DIGITS
         Allow2DigYr% = False

      Case Else
         'we don't allow years greater than 9999
         'year has too many characters
         ErrorFlag% = 401
         GoTo VerifyDate_Error
   End Select

   ErrorFlag% = 500
   
   '** ALLOW FOR TWO DIGIT YEARS IN CURRENT CENTURY ***************
   'At this point, we have our year in a 4 digit format.
   'The following allows programmer to enable/disable TWO digit years
   'for the CURRENT century.
   
   'Set this yourself somewhere in your program
   'or use an IF statement to check a flag somewhere
   'or use as a parameter setting

   'NOTE- the previous SELECT CASE routine checked to see if the user typed
   'a 4-DIGIT year.  If the user had typed a 4-digit year, then Allow2DigYr%
   'will be FALSE and will result in a 4-digit year being returned.
   'If the user typed an entry other than 4-digits, then the return format
   'depends on other parameter settings.
   
   'Allow2DigYr% = True
   
   If Allow2DigYr% Then
   
      'Obtain the century from our date string.
      'We already have our YEAR in 4 digits.
      CenturyVal% = Val(YearTxt$) \ 100

      'Compare the provided century with the current century.
      If CenturyVal% = Year(Now) \ 100 Then
         
         'Convert our YEAR to a two digit year string
         'if the date string century is the same as
         'the current computer system clock century
         YearTxt$ = CStr(Val(YearTxt$) Mod 100)
         YearTxt$ = CStr(Format(YearTxt$, "00"))

         'Added this just for user's assurance that
         'the century rollover will be correct.
         'Any *000 years entered will return *000
         'if the current year is *000
         If Val(YearTxt$) = 0 Then
            If (Year(Now) Mod 100) = 0 Then
               YearTxt$ = CStr(CenturyVal% * 100)
            End If
         End If
      End If
   End If   'Allow2DigYr%

   ErrorFlag% = 600

   'rebuild our date string
   TempDate$ = MonthVal% & "/" & DayVal% & "/" & YearTxt$

   'now check to see if we have a valid date..
   ErrorFlag% = 601
   If Not IsDate(TempDate$) Then GoTo VerifyDate_Error

   'RETURN our reformatted date back
   Cntl = TempDate$
   
   'for rule of single place to exit...
VerifyDate_Exit:
   Exit Sub

'ERROR ROUTINES ***************************************
VerifyDate_Error:
   Select Case ErrorFlag%
      Case 101
         Msg$ = "Only one type of date delimiter allowed."
      Case 102
         Msg$ = "Invalid date format."
      Case 401
         Msg$ = "Year contains too many characters."

      '** UNCOMMENT AS NECESSARY ** SEE PREVIOUS CODE
      '** for "date required" applications...
      'Case 150
      '   Msg$ = "Date field is required."

      Case Else
         Msg$ = "Invalid date. Please enter correct date."

   End Select
   MsgBox Msg$ & NL & "Error: " & ErrorFlag%, 16, "Date Error"
   GoTo VerifyDate_ErrSetFocus

VerifyDate_SysError:
   Msg$ = "Invalid date format. Error: " & Error(Err)
   GoTo VerifyDate_ErrSetFocus

VerifyDate_ErrSetFocus:
   Cntl.SetFocus
   Cntl.SelStart = 0
   Cntl.SelLength = Len(Cntl)
   GoTo VerifyDate_Exit

End Sub

